/*
 * Copyright (C) 2009 Splitted-Desktop Systems. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef VA_GLX_H
#define VA_GLX_H

#ifdef IN_LIBVA
#include "va.h"
#else
#include <va/va.h>
#endif
#include <GL/glx.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Return a suitable VADisplay for VA API
 *
 * @param[in] dpy the X11 display
 * @return a VADisplay
 */
VADisplay vaGetDisplayGLX(
    Display *dpy
);

/**
 * Create a surface used for display to OpenGL
 *
 * The application shall maintain the live GLX context itself.
 * Implementations are free to use glXGetCurrentContext() and
 * glXGetCurrentDrawable() functions for internal purposes.
 *
 * @param[in]  dpy        the VA display
 * @param[in]  target     the GL target to which the texture needs to be bound
 * @param[in]  texture    the GL texture
 * @param[out] gl_surface the VA/GLX surface
 * @return VA_STATUS_SUCCESS if successful
 */
VAStatus vaCreateSurfaceGLX(
    VADisplay dpy,
    GLenum    target,
    GLuint    texture,
    void    **gl_surface
);

/**
 * Destroy a VA/GLX surface
 *
 * The application shall maintain the live GLX context itself.
 * Implementations are free to use glXGetCurrentContext() and
 * glXGetCurrentDrawable() functions for internal purposes.
 *
 * @param[in]  dpy        the VA display
 * @param[in]  gl_surface the VA surface
 * @return VA_STATUS_SUCCESS if successful
 */
VAStatus vaDestroySurfaceGLX(
    VADisplay dpy,
    void     *gl_surface
);

/**
 * Associate a VA surface to a VA/GLX surface
 *
 * The association is live until vaDeassociateSurfaceGLX(),
 * vaCopySurfaceGLX() or the next call to vaBeginPicture() with the
 * specificed VA surface.
 * 
 * The application shall maintain the live GLX context itself.
 * Implementations are free to use glXGetCurrentContext() and
 * glXGetCurrentDrawable() functions for internal purposes.
 *
 * @param[in]  dpy        the VA display
 * @param[in]  gl_surface the VA/GLX surface
 * @param[in]  surface    the VA surface
 * @param[in]  flags      the PutSurface flags
 * @return VA_STATUS_SUCCESS if successful
 */
VAStatus vaAssociateSurfaceGLX(
    VADisplay    dpy,
    void        *gl_surface,
    VASurfaceID  surface,
    unsigned int flags
);

/**
 * Deassociate a VA surface from a VA/GLX surface
 *
 * The application shall maintain the live GLX context itself.
 * Implementations are free to use glXGetCurrentContext() and
 * glXGetCurrentDrawable() functions for internal purposes.
 *
 * @param[in]  dpy        the VA display
 * @param[in]  gl_surface the VA surface
 * @return VA_STATUS_SUCCESS if successful
 */
VAStatus vaDeassociateSurfaceGLX(
    VADisplay dpy,
    void     *gl_surface
);

/**
 * Synchronize a VA/GLX surface
 *
 * This function blocks until all pending operations on the VA/GLX
 * surface have been completed.
 *
 * The application shall maintain the live GLX context itself.
 * Implementations are free to use glXGetCurrentContext() and
 * glXGetCurrentDrawable() functions for internal purposes.
 *
 * @param[in]  dpy        the VA display
 * @param[in]  gl_surface the VA surface
 * @return VA_STATUS_SUCCESS if successful
 */
VAStatus vaSyncSurfaceGLX(
    VADisplay dpy,
    void     *gl_surface
);

/**
 * Prepare VA/GLX surface for rendering
 *
 * This function performs an implicit vaSyncSurfaceGLX().
 *
 * Implementations using the GLX texture-from-pixmap extension will
 * generally call glXBindTexImage() here.
 *
 * The application shall maintain the live GLX context itself.
 * Implementations are free to use glXGetCurrentContext() and
 * glXGetCurrentDrawable() functions for internal purposes.
 *
 * @param[in]  dpy        the VA display
 * @param[in]  gl_surface the VA surface
 * @return VA_STATUS_SUCCESS if successful
 */
VAStatus vaBeginRenderSurfaceGLX(
    VADisplay dpy,
    void     *gl_surface
);

/**
 * Notify the server that the VA/GLX surface is no longer used for
 * rendering
 *
 * Implementations using the GLX texture-from-pixmap extension will
 * generally call glXReleaseTexImage() here.
 *
 * The application shall maintain the live GLX context itself.
 * Implementations are free to use glXGetCurrentContext() and
 * glXGetCurrentDrawable() functions for internal purposes.
 *
 * @param[in]  dpy        the VA display
 * @param[in]  gl_surface the VA surface
 * @return VA_STATUS_SUCCESS if successful
 */
VAStatus vaEndRenderSurfaceGLX(
    VADisplay dpy,
    void     *gl_surface
);

/**
 * Copy a VA surface to a VA/GLX surface
 *
 * This function kills any association that was previously made with
 * vaAssociateSurfaceGLX() and will not return until the copy is
 * completed.
 *
 * Upon successful return, the underlying GL texture will contain the
 * complete pixels and no call to vaBeginRenderSurfaceGLX() or
 * vaEndRenderSurfaceGLX() is required.
 *
 * The application shall maintain the live GLX context itself.
 * Implementations are free to use glXGetCurrentContext() and
 * glXGetCurrentDrawable() functions for internal purposes.
 *
 * @param[in]  dpy        the VA display
 * @param[in]  gl_surface the VA/GLX destination surface
 * @param[in]  surface    the VA source surface
 * @param[in]  flags      the PutSurface flags
 * @return VA_STATUS_SUCCESS if successful
 */
VAStatus vaCopySurfaceGLX(
    VADisplay    dpy,
    void        *gl_surface,
    VASurfaceID  surface,
    unsigned int flags
);

#ifdef __cplusplus
}
#endif

#endif /* VA_GLX_H */
